// bdipole.cpp : Defines the entry point for the console application.
//

//#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <vector>


extern "C" {
int dipole_(double *xq, double *yq, double *zq, 
	        double *a,  double *mi, double *md, double *m, 
	        double *xp, double *yp, double *zp, double *bx, 
	        double *by, double *bz);

}


void output_field(std::vector<double> & x, 
				  std::vector<double> & y,
				  std::vector<double> & z,
				  std::vector<int>    & line,
				  double I, double D,
				  double x0, double y0, double z0,
				  double mi, double md, double m, 
				  double background, double noise_ampl, FILE * out=stdout) {
	
	
	double T1, noise, bx, by, bz, a;
	fprintf(stdout,"#X Y Z T ALT LINE\n");
	
	a = 1.;

	for(int i=0; i<x.size(); i++) {
		
		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x[i], &y[i], &z[i], &bx, &by, &bz);	
		T1 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I)) + background + noise;
		
		if(i && line[i-1]!=line[i]) fprintf(out,"\n");
		fprintf(out, "%lf %lf %lf  0.0 %lf %d\n", y[i], x[i], z[i], T1, line[i]);
		
	}
	
	
}

void output_2_vert_fields(std::vector<double> & x, 
				  std::vector<double> & y,
				  std::vector<double> & z,
				  std::vector<int>    & line,
				  double I, double D,
				  double x0, double y0, double z0,
				  double mi, double md, double m, 
				  double background, double noise_ampl, double separation, FILE * out=stdout) {
	
	
	double T1, T2, noise, bx, by, bz, a, tmp;
	fprintf(stdout,"#X Y Z T_bottom T_top DIFFERENCE ALT LINE\n");
	
	a = 1.;

	for(int i=0; i<x.size(); i++) {
		
		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x[i], &y[i], &z[i], &bx, &by, &bz);	
		T1 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I)) + background + noise;


		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		tmp =  z[i] - separation;
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x[i], &y[i], &tmp, &bx, &by, &bz);	
		T2 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I)) + background + noise;
		
		if(i && line[i-1]!=line[i]) fprintf(out,"\n");
		fprintf(out, "%lf %lf %lf  0.0 %lf %lf %lf %d\n", y[i], x[i], z[i], T1, T2, T1-T2, line[i]);
		
	}
	
	
}


void output_2_vert_fields_comp(std::vector<double> & x, 
				  std::vector<double> & y,
				  std::vector<double> & z,
				  std::vector<int>    & line,
				  double x0, double y0, double z0,
				  double mi, double md, double m, 
				  double background, double noise_ampl, double separation, 
                  int component,
                  FILE * out=stdout) {
	
	
	double T1, T2, noise, bx, by, bz, a, tmp;
	
	switch(component) {
	  case 0: fprintf(stdout,"#X Y Z Z_bottom Z_top DIFFERENCE ALT LINE\n"); break;
	  case 1: fprintf(stdout,"#X Y Z X_bottom X_top DIFFERENCE ALT LINE\n"); break;
	  case 2: fprintf(stdout,"#X Y Z Y_bottom Y_top DIFFERENCE ALT LINE\n"); break;
   }         	    
	
	a = 1.;

	for(int i=0; i<x.size(); i++) {
		
		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x[i], &y[i], &z[i], &bx, &by, &bz);	
		
		switch(component) {
             case 0: T1 = bz; break;             
             case 1: T1 = bx; break;
             case 2: T1 = by; break;                 
         }
		
        T1 += background + noise;


		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		tmp =  z[i] - separation;
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x[i], &y[i], &tmp, &bx, &by, &bz);	
		
		switch(component) {
             case 0: T2 = bz; break;             
             case 1: T2 = bx; break;
             case 2: T2 = by; break;                 
         }
         		
		T2 = background + noise;
		
		if(i && line[i-1]!=line[i]) fprintf(out,"\n");
		fprintf(out, "%lf %lf %lf  0.0 %lf %lf %lf %d\n", y[i], x[i], z[i], T1, T2, T1-T2, line[i]);
		
	}
	
	
}



void output_2_vert_grad(std::vector<double> & x, 
				  std::vector<double> & y,
				  std::vector<double> & z,
				  std::vector<int>    & line,
				  double I, double D,
				  double x0, double y0, double z0,
				  double mi, double md, double m, 
				  double background, double noise_ampl, double separation, FILE * out=stdout) {
	
	
	double T1, T2, noise, bx, by, bz, a, tmp, grad ;
	fprintf(stdout,"#X Y Z Vgrad ALT LINE\n");
	
	a = 1.;
    double s2 = separation/2.;

	for(int i=0; i<x.size(); i++) {

		tmp =  z[i] - s2;
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x[i], &y[i], &tmp, &bx, &by, &bz);	
		T1 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));
		
		tmp =  z[i] + s2;
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x[i], &y[i], &tmp, &bx, &by, &bz);	
		T2 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		grad = (T2 - T1)/separation + background + noise;

		if(i && line[i-1]!=line[i]) fprintf(out,"\n");
		fprintf(out, "%lf %lf %lf  0.0 %lf %d\n", y[i], x[i], z[i], grad, line[i]);
		
	}
	
	
}

void output_vert_magnitude6(std::vector<double> & x, 
				  std::vector<double> & y,
				  std::vector<double> & z,
				  std::vector<int>    & line,
				  double I, double D,
				  double x0, double y0, double z0,
				  double mi, double md, double m, 
				  double background, double noise_ampl, double separation, FILE * out=stdout) {
	
	
	double T, T1, T2, T3, T4, T5, T6, noise, bx, by, bz, a, grad, s2 ;
	double x1, y1, z1, gx, gy, gz;
	fprintf(stdout,"#X Y Z GRAD_MAGNITUDE_6 ALT LINE Gx Gy Gz\n");
	
	a = 1.;

	s2 = separation / 2.;

	for(int i=0; i<x.size(); i++) {
			
		x1 = x[i]; y1 = y[i]; z1 = z[i];
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i] - s2; y1 = y[i]; z1 = z[i];
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T1 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i] + s2; y1 = y[i]; z1 = z[i];
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T2 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i]; y1 = y[i] - s2; z1 = z[i];
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T3 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i]; y1 = y[i] + s2; z1 = z[i];
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T4 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i]; y1 = y[i]; z1 = z[i] - s2;
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T5 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i]; y1 = y[i]; z1 = z[i] + s2;
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T6 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		gx = (T2 - T1)/separation + noise;
		
		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		gy = (T4 - T3)/separation + noise;

		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		gz = (T6 - T5)/separation + noise;

		

		grad = sqrt(gx*gx + gy*gy + gz*gz);

		if(i && line[i-1]!=line[i]) fprintf(out,"\n");

		fprintf(out, "%lf %lf %lf  0.0 %lf %d %lf %lf %lf\n", y[i], x[i], z[i], grad, line[i], gx, gy, gz);
		
	}
	
	
}

void output_vert_magnitude4(std::vector<double> & x, 
				  std::vector<double> & y,
				  std::vector<double> & z,
				  std::vector<int>    & line,
				  double I, double D,
				  double x0, double y0, double z0,
				  double mi, double md, double m, 
				  double background, double noise_ampl, double separation, FILE * out=stdout) {
	
	
	double T, T2, T4, T6, noise, bx, by, bz, a, grad, s2 ;
	double x1, y1, z1, gx, gy, gz;
	fprintf(stdout,"#X Y Z GRAD_MAGNITUDE_4 ALT LINE Gx Gy Gz\n");
	
	a = 1.;

	s2 = separation;

	for(int i=0; i<x.size(); i++) {
			
		x1 = x[i]; y1 = y[i]; z1 = z[i];
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i] + s2; y1 = y[i]; z1 = z[i];
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T2 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i]; y1 = y[i] + s2; z1 = z[i];
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T4 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		x1 = x[i]; y1 = y[i]; z1 = z[i] - s2;
		dipole_(&x0, &y0, &z0, &a, &mi, &md, &m, &x1, &y1, &z1, &bx, &by, &bz);	
		T6 = (bx*cos(I)*cos(D) + by*cos(I)*sin(D) + bz*sin(I));

		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		gx = (T2 - T)/s2 + noise;
		
		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		gy = (T4 - T)/s2 + noise;

		noise = noise_ampl*((double(rand())/RAND_MAX)-0.5);
		gz = (T - T6)/s2 + noise;
		

		grad = sqrt(gx*gx + gy*gy + gz*gz);

		if(i && line[i-1]!=line[i]) fprintf(out,"\n");

		fprintf(out, "%lf %lf %lf  0.0 %lf %d %lf %lf %lf\n", y[i], x[i], z[i], grad, line[i], gx, gy, gz);
		
	}
	
	
}


int main(int argc, char* argv[])
{

	std::vector<double> x, y, z;
	std::vector<int> line;

	char buffer[1024];
	double x1, y1, z1;
	int line1;

	double a  = 1.;
	double m  = 13.59401335854293610023938594399;//10.;
	double mi = 66.65; //45.;//65.6;
	double md = -10.47; //148.98;//-12.27;
	double x0, y0, z0, I, D;
	int n = 0;
    double d2rad = .017453293;
    double background  =0.;
	int is_induced = 0;
	x0 = y0 = z0 = 0;
	double pi = 3.14159265, separation, noise_ampl;;
	int i;
    int type = 0;
	separation = 1.;
	noise_ampl = 0.;

	if(argc == 1) {
		fprintf(stderr,"\nUsage: %s [options] < input > output\n",argv[0]); 
		fprintf(stderr,"-I  number  - Inclination, degrees\n"); 
		fprintf(stderr,"-D  number  - Declination, degrees\n"); 
		fprintf(stderr,"-Bg number  - background value to add (like 45000 nT)\n"); 
		fprintf(stderr,"-mi number  - Magnetic moment inclination, degrees\n"); 
		fprintf(stderr,"-md number  - Magnetic moment declination, degrees\n"); 
		fprintf(stderr,"-Ind        - assume induced mag. only (mi=I, md=D\n"); 
		fprintf(stderr,"-M number   - magnetic moment value, cgs\n"); 
		fprintf(stderr,"-x0 number  - target x-position\n"); 
		fprintf(stderr,"-y0 number  - target y-position\n"); 
		fprintf(stderr,"-z0 number  - target z-position\n"); 
		fprintf(stderr,"-Finite number - Finite separation (default 1 m)\n");
		fprintf(stderr,"-Noise number - add peak to peak random noise (default 0)\n");
		fprintf(stderr,"-Type number - Computed field type\n");
		fprintf(stderr,"           0 - Total magnetic field (default)\n");
		fprintf(stderr,"           1 - Two vertical sensors (needs separation)\n");
		fprintf(stderr,"           2 - Vertical gradient (needs separation)\n");
		fprintf(stderr,"           3 - Gradient magnitude based on 6 sensors (needs separation)\n");
		fprintf(stderr,"           4 - Gradient magnitude based on 4 sensors (needs separation)\n");
		fprintf(stderr,"           5 - Z component of the total field\n");
		fprintf(stderr,"           6 - X component of the total field\n");		
		fprintf(stderr,"           7 - Y component of the total field\n");		
		fprintf(stderr,"Input file has format X, Y, Z, xx xx line\n");  
		exit(0);
	}

	for(i=1; i<argc; i++) {
		if(argv[i][0]=='-') {
			if(strcmp(argv[i],"-I")==0) {
				I = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-D")==0) {
				D = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-mi")==0) {
				mi = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-md")==0) {
				md = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-Bg")==0) {
				background = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-M")==0) {
				m = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-x0")==0) {
				y0 = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-y0")==0) {
				x0 = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-z0")==0) {
				z0 = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-Ind")==0) {
				is_induced = 1; continue;
			}
			if(strcmp(argv[i],"-Type")==0) {
				type = atoi(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-Finite")==0) {
				separation = atof(argv[i+1]); i++; continue;
			}
			if(strcmp(argv[i],"-Noise")==0) {
				noise_ampl = atof(argv[i+1]); i++; continue;
			}
		}
	}

	// report parameters
	FILE * out = stderr;

	for(i=0; i<2; i++) {
    if(i) out = stdout;

	fprintf(out,"# Earth's filed I=%lf D=%lf background=%lf\n", I, D, background);

	if(is_induced)
		fprintf(out,"# Induced moment only\n");
	else
		fprintf(out,"# Dipole mi=%lf md=%lf\n", mi, md);

	fprintf(out,"# Magnetic moment, cgs: %lf\n", m);
	fprintf(out,"# Dipole position x0=%lf y0=%lf z0=%lf\n", y0, x0, z0);
	fprintf(out,"# Noise %lf\n", noise_ampl);
	}



	if(is_induced) {
		mi = I;
		md = D;
	}

	I *= d2rad;
	D *= d2rad;

	// convert cgs to A*m^2 and adjust for volume of the sphere

	m *= (3.e-3/(4*pi));
   
	while(!feof(stdin)) {
		fgets(buffer, 1000, stdin);		
		if(4==sscanf(buffer,"%lf%lf%lf%*s%*s%d", &y1, &x1, &z1, &line1)) {
			x.push_back(x1);
			y.push_back(y1);
			z.push_back(z1);
			line.push_back(line1);
		}
		
	}


	switch(type) {
		
	case 0: 
		fprintf(stderr,"Computing total field\n");
		fprintf(stdout,"#Computing total field\n");
		output_field(x, y, z, line, I, D, x0, y0, z0, mi, md, m, background, noise_ampl);
		break;
		
	case 1: 
		fprintf(stderr,"Computing 2 total fields at vertical separation %lf\n", separation);
		fprintf(stdout,"#Computing 2 total fields at vertical separation %lf\n", separation);
		output_2_vert_fields(x, y, z, line, I, D, x0, y0, z0, mi, md, m, background, noise_ampl, separation);
		break;

	case 2: 
		fprintf(stderr,"Computing vertical gradient at vertical separation %lf\n", separation);
		fprintf(stdout,"#Computing vertical gradient at vertical separation %lf\n", separation);
		output_2_vert_grad(x, y, z, line, I, D, x0, y0, z0, mi, md, m, background, noise_ampl, separation);
		break;

	case 3: 
		fprintf(stderr,"Computing gradient magnitude using 6 sensors with separation %lf\n", separation);
		fprintf(stdout,"#Computing gradient magnitude using 6 sensors with separation %lf\n", separation);
		output_vert_magnitude6(x, y, z, line, I, D, x0, y0, z0, mi, md, m, background, noise_ampl, separation);
		break;
		

	case 4: 
		fprintf(stderr,"Computing gradient magnitude using 4 sensors with separation %lf\n", separation);
		fprintf(stdout,"#Computing gradient magnitude using 4 sensors with separation %lf\n", separation);
		output_vert_magnitude4(x, y, z, line, I, D, x0, y0, z0, mi, md, m, background, noise_ampl, separation);
		break;
	
	case 5: 
		fprintf(stderr,"Computing 2 Z  fields at vertical separation %lf\n", separation);
		fprintf(stdout,"#Computing 2 Z fields at vertical separation %lf\n", separation);
		output_2_vert_fields_comp(x, y, z, line, x0, y0, z0, mi, md, m, background, noise_ampl, separation, 0);
		break;

	case 6: 
		fprintf(stderr,"Computing 2 X  fields at vertical separation %lf\n", separation);
		fprintf(stdout,"#Computing 2 X fields at vertical separation %lf\n", separation);
		output_2_vert_fields_comp(x, y, z, line, x0, y0, z0, mi, md, m, background, noise_ampl, separation, 1);
		break;

	case 7: 
		fprintf(stderr,"Computing 2 Y  fields at vertical separation %lf\n", separation);
		fprintf(stdout,"#Computing 2 Y fields at vertical separation %lf\n", separation);
		output_2_vert_fields_comp(x, y, z, line, x0, y0, z0, mi, md, m, background, noise_ampl, separation, 2);
		break;		
        
        }
            	
	return 0;
}

