//
// @(#)magarray.cpp<2>  1.1  misha-12nov104
//
// Copyright (c) 2004 of Mikhail Tchernychev.
// All rights reserved.
//
// Created:  12nov104  by  misha@misha.local
// Version:  12nov104  by  misha@misha.local
//


#include <iostream>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "magobject.h"
#include "magdipole.h"
#include "magcollection.h"
#include "maggriddata.h"
#include "magprofiles.h"
#include "maginv.h"
#include "magfunc.h"
#include "magarray.h"
#include "magbackground.h"
#include "magbackground2.h"

#include "l2estimator.h"
#include "l1estimator.h"
#include "dnls1estimator.h"
#include "tn_estimator.h"
#include "magchannel.h"

#include "targetarray.h"

using namespace std;


class CLimitedInversion: public CMagInv {
 
public:
    
   double m_lfZmin;
   double m_lfZmax;    
   int m_iCounter;

CLimitedInversion() {    
    m_lfZmin = -1.e+38;
    m_lfZmax = 1.e+38;
    m_iCounter = 0;
};      
    
};    


#include <signal.h>

static int ibreak = 0;

void ctrl_c(int sig) {
  ibreak = 1;
  fprintf(stderr,"signal\n");
}

int progress_func(void * data, int percent, double discr) { // progress / cancel function

  CLimitedInversion * pInversion = (CLimitedInversion *)data;
  double vars[3];

  pInversion->GetNonLinearVariables(vars, 3);

  pInversion->m_iCounter++;
  if(pInversion->m_iCounter < 5) return 0;
  if(vars[2] > pInversion->m_lfZmax || vars[2] < pInversion->m_lfZmin) return 1;  
   
  fprintf(stderr,"iter=%d discrepancy=%lf\n", percent, discr); fflush(stderr);
  return ibreak;

}


main(int argc, char ** argv) {

  signal(SIGINT,ctrl_c);
 
  double positions[3], J[3], max_err, std_err;
 
  double depth   = 2.;
  double I       = 65.6; 
  double D       = -12.27;
  int n_channels = 2;
  double zmin    = 100;
  double zmax    = 100; 
  int background = 1;
  double dm      = 1.e-3;
  int is_shift   = 0;

 
  if(argc == 1) {
    fprintf(stderr,"\nUsage: %s -I val -D val -Ch val  < input > output\n",argv[0]); 
    fprintf(stderr,"-I  number  -  Inclination, degrees\n"); 
    fprintf(stderr,"-D  number  -  Declination, degrees\n"); 
    fprintf(stderr,"-Ch number  - number of channels [2] \n"); 
    exit(0);
  }


for(int i=1; i<argc; i++) {
    if(argv[i][0]=='-') {
      if(strcmp(argv[i],"-I")==0) {
	  I = atof(argv[i+1]); i++; continue;
      }
      if(strcmp(argv[i],"-D")==0) {
 	  D = atof(argv[i+1]); i++;  continue;
      }
      if(strcmp(argv[i],"-Ch")==0) {
   	  n_channels = atol(argv[i+1]); i++; continue;
        }
      }
  }



  // create objects we need in inversion

  CMagProfileArray *pMagArray = new CMagProfileArray;

  if(!pMagArray) {
      fprintf(stderr,"Can not create pMagArray. Stop.\n");
      exit(0);
  }    

  pMagArray->SetChannels(n_channels);

  CMagDipole * pDipole = new CMagDipole(0);
  
  if(!pDipole) {
      fprintf(stderr,"Can not create pDipole. Stop.\n");
      delete pMagArray;
      exit(0);
  }        
  
  // read data
  
  char buffer[1024],  date_txt[20], time_txt[20], name[20];
  const char *IFS = " \t\n";
  double x[10], y[10], z[10], alt[10], T[10];
  char buffer2[1024], buffer1[1024];
     
  fgets(buffer1, 1024, stdin); // skip header
  int k = 0; 
  double x_middle, y_middle, xc, yc, zc, Jx, Jy, Jz;
  int n_data;
              
  fgets(buffer2, 1024, stdin); // data 

  sscanf(buffer2,"%*s%d%*s%*s%lf%lf%lf%lf%lf%lf",
  	 &n_data, &xc, &yc, &zc, &Jx, &Jy, &Jz); 
	 
  sscanf(buffer2,"%*s%d%lf%lf%lf%lf%lf%lf",
	 &n_data, &xc, &yc, &zc, &Jx, &Jy, &Jz); 
	 
  Jx /= 10; Jy /=10; Jz /= 10;
  

  pDipole->FixNonLinearParam(0, FALSE, xc);
  pDipole->FixNonLinearParam(1, FALSE, yc);
  pDipole->FixNonLinearParam(2, FALSE, zc);

  pDipole->FixLinearParam(0, FALSE, Jx);
  pDipole->FixLinearParam(1, FALSE, Jy);
  pDipole->FixLinearParam(2, FALSE, Jz);


 fgets(buffer, 1024, stdin); // skip header
 n_data--;
                   
  for(int j=0; j<n_data; j++) {
      fgets(buffer, 1024, stdin);
      
      char * p  = strtok (buffer, IFS);
                    
      for(int i=0; i<n_channels; i++) {
          x[i]   =  atof(p); p = strtok(NULL, IFS);
          y[i]   =  atof(p); p = strtok(NULL, IFS);
          z[i]   =  atof(p); p = strtok(NULL, IFS);           
          alt[i] =  atof(p); p = strtok(NULL, IFS);
          T[i]   =  atof(p); p = strtok(NULL, IFS);
          p = strtok(NULL, IFS); // skip computed filed                       
      }    
      
      double dtime = 0.;
      strcpy(name,p);     p = strtok(NULL, IFS);
      if(p) {
            strcpy(date_txt,p); p = strtok(NULL, IFS);
            strcpy(time_txt,p); 
            dtime = date2sec(date_txt, time_txt, 1);
      }

      if(!j){
          pMagArray->InitProfile(name);
          pMagArray->SetChannels(n_channels);
      }    
          
     pMagArray->Add(x, y, dtime, T, z, alt);
   }   
 
    
   CMagDataCollection DataCollection;
   CMagCollection     ObjectCollection;
   
   ObjectCollection.AddMagObject(pDipole);
   DataCollection.AddData(pMagArray);
         
     
   CLimitedInversion Inversion;
   Inversion.SetProgressFunc(progress_func);

   Inversion.SetMagData(&DataCollection);   
   Inversion.SetMagCollection(&ObjectCollection);
  
   Inversion.SetEarthFieldDirections(I,D,90.);

   pMagArray->SetStartStop(0, pMagArray->GetTotalTimePoints()-1);

   int n_linear_vars, n_nonline_vars;
   DataCollection.ReScan();
   ObjectCollection.ReScanAllObjects(&n_linear_vars, &n_nonline_vars);


   double * T_synt = new double[n_data*n_channels];
   Inversion.ComputeField(T_synt, n_data*n_channels);
   double fmin, fmax, fampl;
   fmin = 1.e+38;
   fmax = -1.e+38;
   for(int i=0; i<n_channels; i++) {
       for(int j=0; j<n_data; j++) {
           //fprintf(stdout,"%lf\n", T_synt[j*n_channels+i]);
           fmin = min(fmin,T_synt[j*n_channels+i]); 
           fmax = max(fmax,T_synt[j*n_channels+i]); 
       }
   //fprintf(stdout,"\n\n");
   }

   fampl = fabs(fmin-fmax);

 Inversion.DumpObjects(stderr);
  
 fprintf(stderr,"fmin = %lf, fmax=%lf, ampl=%lf\n", fmin, fmax, fampl);

   delete [] T_synt;

   char * p = strstr(buffer1, "DATE"); *(p-1)=0;
   fprintf(stdout,"%s", buffer1);
   fprintf(stdout," AMPL ");
   fprintf(stdout,"%s", p);

   p = strchr(buffer2, '/'); *(p-3)=0;
   p -= 2;
   fprintf(stdout,"%s", buffer2);
   fprintf(stdout," %.2lf ", fampl);
   fprintf(stdout,"%s", p);


} 
