//
// @(#)targetarray.cpp  1.1  misha-12nov104
//
// Copyright (c) 2004 of Mikhail Tchernychev.
// All rights reserved.
//
// Created:  12nov104  by  misha@misha.local
// Version:  12nov104  by  misha@misha.local
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "targetarray.h"
#include "magfunc.h"


_target const & _target::operator=(_target const &o) {

    if (this == &o) return *this;

    for(int i=0; i<3; i++) {
	x[i] = o.x[i];
	J[i] = o.J[i];
	dx[i] = o.dx[i];
	x2[i] = o.x2[i];
    }


    file_handle = o.file_handle;
    fit         = o.fit;
    count       = o.count;
    time        = o.time;
    depth       = o.depth;
    alt         = o.alt;
    n_start     = o.n_start;
    n_stop      = o.n_stop;
    Jtotal      = o.Jtotal; 
    incl        = o.incl;
    decl        = o.incl;
    angle       = o.angle;
    ratio       = o.ratio;
    bg          = o.bg; 
    mag_ampl    = o.mag_ampl;   
    strcpy(line, o.line);
    
    return *this;

}

TARGET * MakeTarget(double *x, double *J, double * dx, unsigned long file_handle,
		    double fit, double time, double depth, double alt,
		    double Jtotal, double incl, double decl, double angle,
		    int n_start, int n_end, double ratio, char * line, double *x2, int bg,
            double mag_ampl) {

    TARGET * pt = new TARGET;
    if(!pt) return NULL;

    for(int i=0; i<3; i++) {
	pt->x[i] = x[i];
	pt->J[i] = J[i];
	pt->dx[i] = dx[i];
	pt->x2[i] = x2[i];
    }

    pt->file_handle = file_handle; 
    pt->fit         = fit;
    pt->count       = 0;
    pt->time        = time;
    pt->depth       = depth;
    pt->alt         = alt;
    pt->n_start     = n_start;
    pt->n_stop      = n_end;
    pt->Jtotal      = Jtotal;
    pt->incl        = incl;
    pt->angle       = angle;
    pt->decl        = decl;
    pt->ratio       = ratio;
    pt->bg          = bg;
    pt->mag_ampl    = mag_ampl;
    strcpy(pt->line, line);

    return pt;

}
		    


void CTargetArray::destroy() {

    int n = m_vTargets.size();
    for(int i=0; i<n; i++) {
	if(m_vTargets[i]) {
	    delete m_vTargets[i];
	    m_vTargets[i] = NULL;
	}
    }

    m_vTargets.clear();

}


CTargetArray::CTargetArray() {
    destroy();
    m_lfMaxCrossDistance = 10.;
    m_lfMaxLongDistance  = 1.;
    m_lfPercent          = 10.;
    m_lfFitTolerance     = 0;
}
    

CTargetArray::~CTargetArray() {
    destroy();
}

CTargetArray::CTargetArray(double cross, double along, double percent, double tolerance) {
    destroy();
    m_lfMaxCrossDistance = cross;
    m_lfMaxLongDistance  = along;
    m_lfPercent          = percent/100.;
    m_lfFitTolerance = tolerance;
}


/*int CTargetArray::AddTarget(double * x, double *J, double nx, double ny,
		   double x0, double y0, double len,
	           double fit, unsigned long fhandle) {


    int n = m_vTargets.size();

    TARGET * pPrevious = NULL; 
    if(n) pPrevious =  m_vTargets[n-1];    

    if(!n || !pPrevious) {
	TARGET * pTarget = new TARGET;
	if(!pTarget) return -1;

	for(int i=0; i<3; i++) {
	    pTarget->x[i] = x[i];
	    pTarget->J[i] = J[i];
	}
	pTarget->file_handle = fhandle;
	pTarget->fit         = fit;
	pTarget->count       = 0;
	m_vTargets.push_back(pTarget);
	return 1;
    }

    double s = fabs((x[0]-x0)*nx + (x[1] -y0)*ny);
    if(s > len*(1.-2*m_lfPercent)) return 0;

    double nx90, ny90;

    nx90 = ny; ny90 = -nx;
    s = fabs((x[0]-x0)*nx90 + (x[1] -y0)*ny90);
    if(s > m_lfMaxCrossDistance) return 0;


    for(int i=0; i<n; i++) {
	pPrevious =  m_vTargets[i];    

	s = fabs((x[0]-pPrevious->x[0])*nx + (x[1] - pPrevious->x[1])*ny);

	if(s <= m_lfMaxLongDistance) {    
	    if(fit < pPrevious->fit) {
		for(int i=0; i<3; i++) {
		    pPrevious->x[i] = x[i];
		    pPrevious->J[i] = J[i];
		}
		pPrevious->file_handle = fhandle;
		pPrevious->fit         = fit;
	    }
	pPrevious->count++;	  
	return 0;
      }	
    }
 

    TARGET * pTarget = new TARGET;
    if(!pTarget) return -1;
    for(int i=0; i<3; i++) {
	pTarget->x[i] = x[i];
	pTarget->J[i] = J[i];
    }
    pTarget->file_handle = fhandle;
    pTarget->fit         = fit;
    m_vTargets[n-1] = pTarget;
    return 0;
   
}

*/

int CTargetArray::AddTarget(CMagInv * pInversion, CMagDipole * pDipole, 
			    CMagProfileArray *pMagArray, 
			    unsigned long fhandle, double ratio, char * line, double * x2, 
                int bg, double mag_ampl) {

    double x[3], J[3], nx, ny, len, dx[3];
    pDipole->GetNonLinearParams(x, 3);
    pDipole->GetNonLinearParams(dx, 3, 1);  
    pDipole->GetLinearParams(J, 3);
    pMagArray->GetNxNy(&nx, &ny);

    double A, B, C, Az, Jtotal, incl, decl, angle;
    pInversion->GetEarthCoeff(&A, &B, &C, &Az);
    pDipole->GetDipoleMoment(A, B, C, Az, &Jtotal, &incl, &decl, &angle);

    len = pMagArray->GetAreaLength();

    double x0, y0, z0, alt0;
    pMagArray->GetMiddlePointCoords(&x0, &y0, &z0, &alt0);

    double median_depth = pMagArray->GetMedianDepth();
    double median_alt   = pMagArray->GetMedianAlt();
    double dtime        = pMagArray->GetNearestTime(x[0], x[1]);

    int n_start, n_stop;
    pMagArray->GetStartStop(&n_start, &n_stop);

    double fit = pInversion->GetNonLinearDiscrepancy();


    int n = m_vTargets.size();

    TARGET * pPrevious = NULL; 
    if(n) pPrevious =  m_vTargets[n-1];    
    if(!n || !pPrevious) {
	TARGET * pTarget = MakeTarget(x, J, dx, fhandle, fit, dtime, median_depth, median_alt,
			     Jtotal, incl, decl, angle, n_start, n_stop, ratio, line, x2, bg, mag_ampl);
	
	if(!pTarget) return -1;

	m_vTargets.push_back(pTarget);
	return 1;
    }

    double s = fabs((x[0]-x0)*nx + (x[1] -y0)*ny);
    if(s > len*(1.-2*m_lfPercent)) return 0;

    double nx90, ny90;

    nx90 = ny; ny90 = -nx;
    s = fabs((x[0]-x0)*nx90 + (x[1] -y0)*ny90);
    if(s > m_lfMaxCrossDistance) return 0;


    for(int i=0; i<n; i++) {
	pPrevious =  m_vTargets[i];    

	s = fabs((x[0]-pPrevious->x[0])*nx + (x[1] - pPrevious->x[1])*ny);

	if(s <= m_lfMaxLongDistance) {    
	    if(fabs(fit - pPrevious->fit) > m_lfFitTolerance) {
		if(fit < pPrevious->fit) {
		TARGET * pTarget = MakeTarget(x, J, dx, fhandle, fit, dtime, median_depth, median_alt,
					      Jtotal, incl, decl, angle, n_start, n_stop, ratio, line, x2, bg, mag_ampl);
		if(!pTarget) return -1;
		pTarget->count = pPrevious->count;
		*pPrevious = *pTarget;
		delete pTarget;
	     }
	    }
	    else {
		if(Jtotal < pPrevious->Jtotal) {
		TARGET * pTarget = MakeTarget(x, J, dx, fhandle, fit, dtime, median_depth, median_alt,
					      Jtotal, incl, decl, angle, n_start, n_stop, ratio, line, x2, bg, mag_ampl);
		if(!pTarget) return -1;
		pTarget->count = pPrevious->count;
		*pPrevious = *pTarget;
		delete pTarget;
		}
	    }

	pPrevious->count++;
	return 0;
      }	
    }
 

    TARGET * pTarget = MakeTarget(x, J, dx, fhandle, fit, dtime, median_depth, median_alt,
 			        Jtotal, incl, decl, angle, n_start, n_stop, ratio, line, x2, bg, mag_ampl);
    if(!pTarget) return -1;

    m_vTargets.push_back(pTarget);
    return 0;

}



int CTargetArray::GetTarget(int n_target, double *x, double *J, unsigned long *fhandle, double * fit){

    int n = m_vTargets.size();
    if(n_target<0 || n_target>=n) return 0;

    TARGET * pTarget = m_vTargets[n_target]; 
    if(!pTarget) return 0;

    for(int i=0; i<3; i++) {
	x[i] = pTarget->x[i];
	J[i] = pTarget->J[i];
    }

    *fhandle = pTarget->file_handle;
    *fit     = pTarget->fit;

    return 1;

}




void CTargetArray::PrintShortList(FILE * dat, FILE * binary_in, FILE * field_out) {

    char date_txt[20], time_txt[20];
    fprintf(dat,"X Y Z IR Jtotal FIT COUNT MAGDEPTH MAGALT START STOP DATE TIME S1 S2\n");

    int n = m_vTargets.size();
    
    int n_start = 0, ret;

    for(int i=0; i<n; i++) {
	TARGET * p = m_vTargets[i];

	sec2date (p->time, date_txt, time_txt, 1);

	ret = DumpBinaryOut(p->file_handle, binary_in, field_out, n_start);

	fprintf(dat,"%.2lf %.2lf %.2lf %lf %lg %lg %d %.2lf %.2lf %d %d %s %s ",
		p->x[0], p->x[1], p->x[2], p->ratio, p->Jtotal*10, p->fit, p->count,
		p->depth, p->alt, p->n_start, p->n_stop, date_txt, time_txt);
     
    fprintf(dat,"%d %d\n", n_start, n_start+ret);
    n_start += ret;  
    n_start++;
   }    

}


void CTargetArray::PrintAll(FILE * dat, int n_data) {

    char date_txt[20], time_txt[20];
    
    //fprintf(dat,"X Y Z IR Jtotal FIT COUNT MAGDEPTH MAGALT START STOP DATE TIME S1 S2\n");

    //#P N_DATA X Y Z Jx Jy Jz FIT MASS Jtotal INCL DECL MAGDEPTH MAGALT DATE TIME LINE
   
   
    fprintf(dat,"#HEADER: #P N_DATA X Y Z Jx Jy Jz FIT MASS Jtotal INCL DECL ANGLE DX DY DZ MAGDEPTH MAGALT RATIO SX SY BG AMPL(nT) DATE TIME LINE\n");
   
    int n = m_vTargets.size();
    fprintf(stderr,"n=%d\n", n);
    
    int n_start = 0, ret;

    for(int i=0; i<n; i++) {
	TARGET * p = m_vTargets[i];

	sec2date (p->time, date_txt, time_txt, 1);

	fprintf(dat,"#P %d %.2lf %.2lf %.2lf %lg %lg %lg %lf %.1lf %lg %.2lf %.2lf %.2lf %.2lf %.2lf %.2lf %.2lf %.2lf %lf %.2lf %.2lf %d %.3lf %s %s %s\n",
		n_data, p->x[0], p->x[1], p->x[2], p->J[0]*10, p->J[1]*10, p->J[2]*10,
		p->fit, (p->Jtotal*10)/5.e+2, p->Jtotal*10, p->incl, p->decl, p->angle, p->dx[0], p->dx[1], p->dx[2],
		p->depth, p->alt, p->ratio, p->x2[0], p->x2[1], p->bg, p->mag_ampl, date_txt, time_txt, p->line);    
   }    
}

int DumpBinaryOut(unsigned long fhandle, FILE * in, FILE *out, int n_start) {
    
    int n_strings = 0;
    int i,j,k;
    
    char date_txt[20], time_txt[20], line[20];
    
    fseek(in, fhandle, SEEK_SET);
    
    int n_channels, n;
    
    fread(&n, 1, sizeof(int), in);
    fread(&n_channels, 1, sizeof(int), in);
    
    fprintf(out,"#");
    for(i=0; i<n_channels; i++) {
        k = i+1;
        fprintf(out,"X%d Y%d Z%d ALT%d T%d TSYNT%d ", k,k,k,k,k,k,k);
    }
    
    fprintf(out,"DATE TIME LINE\n");
     
    double a; 
    for(i=0; i<n; i++) {
      for(j=0; j<n_channels; j++) {
       fread(&a, 1, sizeof(double),in); fprintf(out,"%.2lf ", a);
       fread(&a, 1, sizeof(double),in); fprintf(out,"%.2lf ", a);
       fread(&a, 1, sizeof(double),in); fprintf(out,"%.2lf ", a);
       fread(&a, 1, sizeof(double),in); fprintf(out,"%.2lf ", a);
       fread(&a, 1, sizeof(double),in); fprintf(out,"%.2lf ", a);
       fread(&a, 1, sizeof(double),in); fprintf(out,"%.2lf ", a);
   }        
      fread(&a, 1, sizeof(double),in);
      sec2date(a, date_txt, time_txt, 1); fprintf(out,"%s %s ", date_txt, time_txt);   
      fread(line, 1, 20, in);
      fprintf(out,"%s\n", line);           
   } 
   
   
    return  n + 1;             
    
}    
    
    



